local addonName, ns = ...
local L = ns.L
local amHunter = select(2,UnitClass("player")) == "HUNTER"

local MAX_PETS = (NUM_PET_ACTIVE_SLOTS + NUM_PET_STABLE_PAGES * NUM_PET_STABLE_SLOTS)

local EventFrame = CreateFrame("Frame")

-- per-char saved vars
StableHandPerChar = StableHandPerChar or {}

-- maps pet family icons to english pet family names (eliminates need for translations)
local petIconMap = {
  [877476] = "Basilisk",
  [132182] = "Bat",
  [132183] = "Bear",
  [133570] = "Beetle",
  [132192] = "Bird of Prey",
  [132184] = "Boar",
  [132200] = "Carrion Bird",
  [132185] = "Cat",
  [236190] = "Chimaera",
  [1044794] = "Clefthoof",
  [236191] = "Core Hound",
  [132186] = "Crab",
  [877479] = "Crane",
  [132187] = "Crocolisk",
  [236192] = "Devilsaur",
  [877480] = "Direhorn",
  [877481] = "Dog",
  [132188] = "Dragonhawk",
  [929300] = "Feathermane",
  [45822] = "Fox",
  [877477] = "Goat",
  [132189] = "Gorilla",
  [463493] = "Hydra",
  [132190] = "Hyena",
  [2011146] = "Krolusk",
  [2027936] = "Lizard",
  [132247] = "Mechanical",
  [877482] = "Monkey",
  [236193] = "Moth",
  [132191] = "Nether Ray",
  [616693] = "Oxen",
  [1624590] = "Pterrordax",
  [625905] = "Quilen",
  [132193] = "Raptor",
  [132194] = "Ravager",
  [1044490] = "Riverbeast",
  [644001] = "Rodent",
  [646378] = "Scalehide",
  [132195] = "Scorpid",
  [136040] = "Serpent",
  [877478] = "Shale Spider",
  [236195] = "Silithid",
  [132196] = "Spider",
  [236165] = "Spirit Beast",
  [132197] = "Sporebat",
  [1044501] = "Stag",
  [132198] = "Tallstrider",
  [804969] = "Toad",
  [132199] = "Turtle",
  [132201] = "Warp Stalker",
  [236196] = "Wasp",
  [643423] = "Water Strider",
  [132202] = "Wind Serpent",
  [132203] = "Wolf",
  [236197] = "Worm",
}
-- which classes are exotic-only
local exotics = {
  ["Shale Spider"] = true,
  ["Silithid"] = true,
  ["Water Strider"] = true,
  ["Quilen"] = true,
  ["Core Hound"] = true,
  ["Devilsaur"] = true,
  ["Spirit Beast"] = true,
  ["Chimaera"] = true,
  ["Worm"] = true,
  ["Clefthoof"] = true,
  ["Krolusk"] = true,
  ["Pterrordax"] = true
}

-- specs, colorcoded. --gwiezdoblask
local Fer = "|cFFCC0000" .. L["Ferocity"] .. "|r"
local Cun = "|cFF44AA08" .. L["Cunning"] .. "|r"
local Ten = "|cFF1070FF" .. L["Tenacity"] .. "|r"
local unknown_spec_string = "|cFFFF88FF" .. L["Unknown spec"] .. "|r"

-- what buffs/abilities each family has
local buffs = {
  ["Basilisk"] =        { Cun, L["Snare"] },
  ["Bat"] =             { Fer, L["Tranq/Dispel"] },
  ["Bear"] =            { Ten, L["Shield Wall"] .. L[" (auto)"] },
  ["Beetle"] =          { Ten, L["Shield Wall"] .. L[" (toggle)"] },
  ["Bird of Prey"] =    { Cun, L["Snare"] },
  ["Boar"] =            { Cun, L["Shield Wall"] .. L[" (toggle)"] },
  ["Carrion Bird"] =    { Fer, L["Mortal Wounds"] },
  ["Cat"] =             { Fer, L["Dodge"] .. ", " .. L["Prowl"] },
  ["Chimaera"] =        { Fer, L["Snare"] .. L[" (ranged)"] .. ", " .. L["AOE"]},
  ["Clefthoof"] =       { Fer, L["Shield Wall"] .. L[" (auto)"] .. ", " .. L["Tanky"] },
  ["Core Hound"] =      { Fer, L["Shield Wall"] .. L[" (toggle)"] ..", " .. L["Thorns"] },
  ["Crab"] =            { Ten, L["Snare"] },
  ["Crane"] =           { Ten, L["Tranq/Dispel"] },
  ["Crocolisk"] =       { Fer, L["Snare"] },
  ["Devilsaur"] =       { Fer, L["Mortal Wounds"] .. ", " .. L["Cannibalize"] },
  ["Direhorn"] =        { Ten, L["Mortal Wounds"] },
  ["Dog"] =             { Cun, L["Snare"] },
  ["Dragonhawk"] =      { Ten, L["Dodge"] },
  ["Feathermane"] =     { Ten, L["Dodge"] .. ", " .. L["Slowfall"] },
  ["Fox"] =             { Cun, L["Dodge"] },
  ["Goat"] =            { Cun, L["Shield Wall"] .. L[" (auto)"] },
  ["Gorilla"] =         { Fer, L["Shield Wall"] .. L[" (auto)"] },
  ["Hydra"] =           { Ten, L["Mortal Wounds"] },
  ["Hyena"] =           { Cun, L["Mortal Wounds"] },
  ["Krolusk"] =         { Ten, L["Shield Wall"] .. L[" (toggle)"] .. " ," .. L["Physical Resistance"] },
  ["Lizard"] =          { Ten, L["Mortal Wounds"] },
  ["Mechanical"] =      { Cun, L["Shield Wall"] .. L[" (toggle)"] },
  ["Monkey"] =          { Cun, L["Dodge"] },
  ["Moth"] =            { Cun, L["Tranq/Dispel"] },
  ["Nether Ray"] =      { Fer, L["Tranq/Dispel"] },
  ["Oxen"] =            { Ten, L["Shield Wall"] .. L[" (auto)"] },
  ["Pterrordax"] =      { Cun, L["Shield Wall"] .. L[" (auto)"] .. ", " .. L["Slowfall"] },
  ["Quilen"] =          { Ten, L["Shield Wall"] .. L[" with Self-Heal"] .. L[" (auto)"] .. ", " .. L["Reincarnation"] },
  ["Raptor"] =          { Cun, L["Mortal Wounds"] },
  ["Ravager"] =         { Fer, L["Mortal Wounds"] },
  ["Riverbeast"] =      { Ten, L["Mortal Wounds"] },
  ["Rodent"] =          { Cun, L["Mortal Wounds"] },
  ["Scalehide"] =       { Fer, L["Shield Wall"] .. L[" (toggle)"] },
  ["Scorpid"] =         { Fer, L["Mortal Wounds"] },
  ["Serpent"] =         { Cun, L["Dodge"] },
  ["Shale Spider"] =    { Cun, L["Shield Wall"] .. L[" (toggle)"] .. ", " .. L["Magic Resistance"] },
  ["Silithid"] =        { Cun, L["Snare"] .. ", " .. L["Fast"] },
  ["Spider"] =          { Fer, L["Snare"] },
  ["Spirit Beast"] =    { Ten, L["Tranq/Dispel"] .. ", " .. L["Prowl"].. ", " .. L["Heal"]},
  ["Sporebat"] =        { Cun, L["Tranq/Dispel"] },
  ["Stag"] =            { Ten, L["Tranq/Dispel"] },
  ["Tallstrider"] =     { Fer, L["Snare"] },
  ["Toad"] =            { Ten, L["Dodge"] },
  ["Turtle"] =          { Ten, L["Shield Wall"] .. L[" (toggle)"] },
  ["Warp Stalker"] =    { Cun, L["Snare"] },
  ["Wasp"] =            { Fer, L["Mortal Wounds"] },
  ["Water Strider"] =   { Cun, L["Tranq/Dispel"] .. ", " .. L["Water Walking"]},
  ["Wind Serpent"] =    { Fer, L["Dodge"] },
  ["Wolf"] =            { Fer, L["Snare"] },
  ["Worm"] =            { Ten, L["Snare"] .. L[" (ranged)"] .. ", " .. L["AOE"]},
}

-- maps call pet spellid to petslot index
local callPetSpells = {
  [883] = 1,
  [83242] = 2,
  [83243] = 3,
  [83244] = 4,
  [83245] = 5,
}

-- returns new tooltip information for a given pet stable slot
local function GetNewTooltip(petSlot)
  local icon, name, level, family = GetStablePetInfo(petSlot)
  if not family or family == "" then return "" end
  
  local englishfamily = petIconMap[icon]
  if not englishfamily then englishfamily = "Unknown" end
  
  -- handle missing families
  if not buffs[englishfamily] then buffs[englishfamily] = {} end
  
  local newfamily = family
  -- add " (Exotic)" to pet family display
  if exotics[englishfamily] then
    newfamily = newfamily.." |cFFFFFF20"..L["(Exotic)"].."|cFFFFFFFF"
  end

  -- start off with default tooltip
  local tooltipstr = format(STABLE_PET_INFO_TOOLTIP_TEXT, level, newfamily, GetSpecString(family)).."|n"

  -- add buffs/abilities
  tooltipstr = tooltipstr.."|cFFFFFFFF"..L["Abilities"]..": ".. GetAbilityString(family)

  return tooltipstr
end

--add info to stabled pet tooltips
hooksecurefunc("PetStable_UpdateSlot", function (button, petSlot)
  -- is there actually a pet in this slot?
  local _, _, _, family = GetStablePetInfo(petSlot)
  if not family or family == "" then return end
  
  -- set button tooltip
  button.tooltipSubtext = GetNewTooltip(petSlot)
  
  if GameTooltip:IsOwned(button) then button:GetScript("OnEnter")(button) end
end)

-- add info to Call Pet # tooltips
GameTooltip:HookScript("OnTooltipSetSpell", function (tooltip)
  local spellName, spellID = tooltip:GetSpell()
  if not spellName then return end
  
  -- is this a "call pet" spell?
  local petSlot = callPetSpells[spellID]
  if not petSlot then return end
  
  -- is there actually a pet in this slot?
  local _, _, _, family = GetStablePetInfo(petSlot)
  if not family or family == "" then return end

  -- add to spell tooltip
  tooltip:AddLine("|n|cFFFFFFFF"..GetNewTooltip(petSlot))
end)


--[[
--    Everything above this point was originally in StableHelper v2.2, by cybermind
--    Some of it has been altered.
]]    

function GetSpecString(family)
    if buffs[family] then
        return buffs[family][1];
    else 
        return unknown_spec_string;
    end    
end   

function GetAbilityString(family)
    if buffs[family] then
        return "|cFFFFD200" .. buffs[family][2] .. "|r"
    else
        return "|cFFFF88FF" .. L["No abilities"] .. "|r"
    end
end    

local function colorizeSpeed(sec)
    if not(sec) then return "" end
    if (sec < 0.25) then  -- capped, magenta
        return "|cFFFF00FF"
    elseif (sec <= 0.5 ) then -- very fast, red
        return "|cFFFF0000"
    elseif (sec <= 1 ) then -- quite fast, orange
        return "|cFFFF6600"
    elseif (sec <=1.8) then -- some fast, yellow
        return "|cFFFFFF00"
    elseif (sec == 2) then -- typical npc attack speed, whitish
        return "|cFFDDDDDD"

    elseif (sec >= 3.5) then -- slooooow, blue
        return "|cFF0088FF"
    elseif (sec >= 2.8) then -- some slow, cyan
        return "|cFF00CC88"
    else -- sec > 2 -- a little slow, pastel green
        return "|cFF88DDAA"
    end
end

GameTooltip:HookScript("OnTooltipSetUnit", function (tooltip)
    local ttUnit = select(2,tooltip:GetUnit())
    if not(ttUnit) then return end
    local unitType = UnitCreatureType(ttUnit)
    local unitFam = UnitCreatureFamily(ttUnit)

    -- show info if unit is from a tameable family AND
    -- 1. player is a hunter OR
    -- 2. unit is another player's pet
    if ( (amHunter or UnitIsOtherPlayersPet(ttUnit)) and
        unitFam and (unitType == "Beast" or unitType == "Mechanical")) then
        tooltip:AddLine(GetSpecString(unitFam) .. "|n|cFFFFFFFF"..L["Abilities"]..": ".. GetAbilityString(unitFam))
    end

    if not (StableHandPerChar.showAtkSpd and 
            unitFam and (unitType == "Beast" or unitType == "Mechanical")) then
        return
    end
    local unitAtkSpd = UnitAttackSpeed(ttUnit)
    local baseAtkSpd
    if (amHunter and ttUnit == "pet") then
        baseAtkSpd = floor(unitAtkSpd * (100+GetPetMeleeHaste())+0.5)/100
    end
    unitAtkSpd = floor(unitAtkSpd*100+0.5)/100
    tooltip:AddDoubleLine(L["Attack Speed"]..":",
        colorizeSpeed(unitAtkSpd)..tostring(unitAtkSpd).."|r"..
            ((amHunter and 
              colorizeSpeed(baseAtkSpd).." ("..tostring(baseAtkSpd)..")".."|r") 
            or ""), 0.8,0.8,0.8, 0.8,0.8,0.8)
end)    

local function toggleAttackSpeed()
    StableHandPerChar.showAtkSpd = not StableHandPerChar.showAtkSpd;
    ChatFrame1:AddMessage( 
        (StableHandPerChar.showAtkSpd and ("|cFF88FF88" .. L["Show"]) or
        ("|cFF880000"..L["Hide"]))  
        .. " ".. L["Attack Speed"] .. "|r");
end    
    
SlashCmdList["STABLEHAND"] = toggleAttackSpeed;
SLASH_STABLEHAND1 = "/stablehand";
